function [deltax,deltay,deltaz,deltar]=CRLB_point(x,y,z,r,pix,lambda,sizeH,FHc,sigm,n,A)

%-------------------------------------------------------------------------%
% Programmed by: Mozhdeh Seifi - Laboratoire Hubert Curien 
%                               http://laboratoirehubertcurien.fr/
%
% Contact : corinne.fournier@univ_st_etienne.fr
% Sept 2011
%-------------------------------------------------------------------------%
%
% Goal : Computing the DH reconstruction theoretical accuracy (deltax,deltaz,deltar) in the sensor center of a spherical particle (parameters (z,r).
% This algorithm uses numerical computation of CRLB for a spherical
% particle.
%-------------------------------------------------------------------------%
% inputs:  
% x : vertical coordinate of the particle on sensor (unit meters)
% y : horizontal coordinate of the particle on sensor (unit meters)
% z : depth coordinate of the object (unit meters)
% r : radius of the object (unit meters)
% pix : pixel size (unit meters)
% lambda : wavelenght (unit meters)
% sizeH : row vector of 2 columns which defines the hologram's size in vertical and horizontal axes respectively
% FHc : give the ratio between map size and hologram size (e.g. if FHc=2 map size is 2 time the holo s
% sigm : the standard deviation of the noise (assume to be white and gaussian)
% n : (downsampling) ? (downsampling rate in pixels : fill-factor) 
% A is the peak amplitude (half of the Peak to peak amplitude) 
% if A =0 , signal aplitude is considered to be =pi*r*r/2/lambda/z 
% if A~=0 , the A value is taken into account 
%-------------------------------------------------------------------------%
% outputs: 
% deltax,deltay,deltaz,deltar : theoretical accuracy in the center of the sensor (unit meters)
%-------------------------------------------------------------------------%
% Test: 
% pix=6.7e-6;z=0.25;r=45e-6;lambda=0.532e-6;sizeH=[1024,1024];sigm=0.05;A=10;
% FHc=[1 1];n=sqrt(0.7);x=0;y=0;
% [deltax,deltay,deltaz,deltar]=CRLB_point(x,y,z,r,pix,lambda,sizeH,FHc,sigm,n,A);
%!-------------------------------------------------------------------------%
%! processing time : ~1.09 s on Cetautomatix . The bottle neck is the gradients
%! computations and more specificely the computations of sinc and bessel cardinal functions 
%! profiler : CRLB_point=0.016s, Gradient_filtering=0.519s, sinc=0.308s , j0c=0.114s, j1c=0.108s 
%-------------------------------------------------------------------------%
% functions used : Gradients_filtering.m, j1c.cpp, j0c.cpp
%-------------------------------------------------------------------------%
% [1] C. Fournier, L. Denis, et T. Fourner, "On the single point resolution of on-axis digital holography",
% Journal of the Optical Society of America A, vol. 27, no. 8, p.
% 1856-1862, 2010.
%-------------------------------------------------------------------------%

if max(size(sizeH))==1
    sizeH=repmat(sizeH,[1 2]);
end

[Gx,Gy,Gz,GR,G]=Gradients_filtering(pix,x,y,z,r,lambda,sizeH,FHc,n);
clear G

if n>1
    sizeH=sizeH./n;
end


I=zeros(4,4);
[s1 s2]=size(Gx);

Gx=Gx(floor(sizeH(1)/2)+1:s1-floor(sizeH(1)/2),floor(sizeH(2)/2)+1:s2-floor(sizeH(2)/2));Gx=Gx(:);
Gy=Gy(floor(sizeH(1)/2)+1:s1-floor(sizeH(1)/2),floor(sizeH(2)/2)+1:s2-floor(sizeH(2)/2));Gy=Gy(:);
Gz=Gz(floor(sizeH(1)/2)+1:s1-floor(sizeH(1)/2),floor(sizeH(2)/2)+1:s2-floor(sizeH(2)/2));Gz=Gz(:);
GR=GR(floor(sizeH(1)/2)+1:s1-floor(sizeH(1)/2),floor(sizeH(2)/2)+1:s2-floor(sizeH(2)/2));GR=GR(:);

I(1,1)=Gx'*Gx;
I(1,2)=Gx'*Gy;
I(1,3)=Gx'*Gz;
I(1,4)=Gx'*GR;

I(2,1)=I(1,2);
I(3,1)=I(1,3);
I(4,1)=I(1,4);

I(2,2)=Gy'*Gy;
I(2,3)=Gy'*Gz;
I(2,4)=Gy'*GR;

I(3,2)=I(2,3);
I(4,2)=I(2,4);

I(3,3)=Gz'*Gz;
I(3,4)=Gz'*GR;
I(4,3)=I(3,4);

I(4,4)=GR'*GR;

I=(I/sigm/sigm);
if A~=0
    A0=pi*r*r/2/lambda/z;
    I=I/(A0*A0)*A*A;
    
end


C=inv(I);
deltax=sqrt(C(1,1));
deltay=sqrt(C(2,2));
deltaz=sqrt(C(3,3));
deltar=sqrt(C(4,4));

