function [Gxa,Gya,Gza,GRa,G]=Gradients_filtering(pixel_size,xn,yn,z,R,lambda,th,FHc,n)
%-------------------------------------------------------------------------%
% Programmed by: Mozhdeh Seifi - Laboratoire Hubert Curien 
%                               http://laboratoirehubertcurien.fr/
% 
% Contact : corinne.fournier@univ_st_etienne.fr
% Sept 2011
%-------------------------------------------------------------------------%
%% This function computes the gradients of the model (see JOSAA2007a); 
%% INPUT parameters :
% pixel_size : pixel size in meters(original holo)
% xn,yn : vertical and horizontal coordinates of object in meters
% z : distance between the particle and the sensor in meters
% R : particle radius in meters
% lambda : wavelenght in meters
% th : hologram size in meters(original holo)
% FHc : give the ratio between map size and hologram size (e.g. if FHc=2 map size is 2 times the holo size)
% n :(downsampling) ? (downsampling rate in pixels : fill-factor) 

%% OUTPUT parameters :
% Gxa, Gya, Gza, GRa : Gradient of the model versus x y z R, units= meters^-1
% G : model (size tm=(FHc+1)th), no units 

%% Test
%  pixel_size=6.7e-6;xn=0;yn=0;z=0.25;R=45e-6;lambda=0.532e-6;th=512;FHc=2;k=sqrt(0.7);
% [Gxa,Gya,Gza,GRa,G]=Gradients_filtering(pixel_size,0,0,z,R,lambda,th,FHc,k);
%!-------------------------------------------------------------------------%
%! processing time : ~0.505 s on Cetautomatix .  
%! profiler : Gradients_filtering=0.238s, sinc=0.162s(5 calls), j0c=0.043s, j1c=0.043s ...
%-------------------------------------------------------------------------%
% functions used : j1c.cpp, j0c.cpp
%-------------------------------------------------------------------------%
%-------------------------------------------------------------------------%
% [1] F. Soulez, L. Denis, C. Fournier, E. Thiebaut et C. Goepfert, "Inverse-problem approach for particle digital
%  holography: accurate location based on local optimization", Journal of the Optical Society of America A, vol. 24, no. 4, p. 1164-1171, 2007
%-------------------------------------------------------------------------%

if size(th,2)==1
    th=repmat(th,[1 2]);
end
if size(FHc,2)==1
    FHc=repmat(FHc,[1 2]);
end

tc=FHc.*th;% map size
tm=tc+th;% model size

[Yk,Xk]=meshgrid(pixel_size*(-tm(2)/2:tm(2)/2-1),pixel_size*(-tm(1)/2:tm(1)/2-1)); %

 % pixels grid for the model(m) % 0 is on the pixel positioned on the tm/2+1 sample  
%% Computation of the model and of the analytical gradient Gxa,Gya,Gza,GRa
Rho2=(xn-Xk).^2+(yn-Yk).^2;
Rho=sqrt(Rho2);
lz=lambda*z;
% L=th*pixel_size;
the=pi*Rho2/(lz);
phi=2*pi*Rho*R/(lz);

clear Rho

tmpsc=sinc(the/pi); 
% tmpj1c=Rho2;

%% if you don't have the mex file for J1c, use the commneted else part
% if BMex==1
    tmpj1c=j1c(phi);
% else
%     ind=find(phi~=0);
%     tmpj1c(ind)=besselj(1,phi(ind))./phi(ind);
%     ind0=find(phi==0);
%     if isempty(ind0)~=1
%         tmpj1c(ind0)=0.5;
%     end
% end



%% MODEL
G=((pi*R^2)/(2*lz))*2*tmpj1c.*tmpsc.*the; % no units

%% if you don't have the mex file for J0c, use the commneted else part
% if BMex==1
    tmpj0=j0c(phi).*phi;
% else
%     tmpj0=besselj(0,phi);
% end

clear phi

%% R GRADIENT JOSAa 
cteR=(pi*R)/(lz);
GRa=cteR*tmpsc.*the.*tmpj0;% unit=m^-1
tmpc=cos(the);

clear the
%% z GRADIENT JOSAa
ctez=-(pi^2)*(R^2)/(lz^2)/z;
Gza=ctez*Rho2.*[((tmpj0-tmpj1c).*tmpsc)+ (tmpj1c.*tmpc)];

if n>0 % fill factor or rate of down_sampling in pixels
   
    nlz=n*pixel_size/lz;
    sincX=sinc(nlz.*Xk);
    sincY=sinc(nlz.*Yk);

    Gza=Gza.*sincX.*sincY + G/z .* ( sincX .*(-cos(nlz*pi.*Yk) + sincY) + sincY .*(-cos(nlz*pi.*Xk) + sincX) );
    GRa=GRa.*sincX.*sincY;
    if n>1% case of down_sampling
        Gza=Gza(1:n:end,1:n:end);
        GRa=GRa(1:n:end,1:n:end);
    end
end

clear Rho2
%% x y GRADIENTs JOSAa
ctex=pi^2*R*R/(lz^2);
tmp=   (tmpsc.*   (tmpj0-2.*tmpj1c)  )    +    (2.*tmpj1c.*tmpc);% dernier terme preprondérant

if n>0 
    xnan=Xk;
    if xnan(floor(end/2)+1,:)==0 
        xnan(floor(end/2)+1,:)=eps;
    end
    
    ynan=Yk;
    if ynan(:,floor(end/2)+1)==0%
        ynan(:,floor(end/2)+1)=eps;
    end
    
     sincX=sinc(nlz.*xnan);
     sincY=sinc(nlz.*ynan);
    Gxa=sincY .* ( sincX .* (ctex*(xn-xnan).*tmp) + G .* ( cos(pi.*xnan.*nlz)-sincX )./xnan);% unit=m-1
    if n>1% case of down_sampling
        Gxa=Gxa(1:n:end,1:n:end);
    end
    
    Gya=sincX .* ( sincY .* (ctex*(yn-ynan).*tmp) + G .* ( cos(pi.*ynan.*nlz)-sincY )./ynan);% unit=m-1
    if n>1% case of down_sampling
        Gya=Gya(1:n:end,1:n:end);
    end
else
    Gxa=ctex*(xn-Xk).*tmp;% unit=m-1
    Gya=ctex*(yn-Yk).*tmp;% unit=m-1
end


clear tmpj1c tmpc tmpsc tmpj0
clear tmp sincX sincY
