function precise=local_optim(hol,found,lambda,pixel,w1,FOV_size1,FOV_size2,I0,K,factors,ranges,SecondT,MEXmodelFAST)
%-------------------------------------------------------------------------%
% Programmed by: Mozhdeh Seifi - Laboratoire Hubert Curien 
%                               http://laboratoirehubertcurien.fr/
% 
% Contact : corinne.fournier@univ_st_etienne.fr
% Sept 2011
%-------------------------------------------------------------------------%
%-------------------------------------------------------------------------%
% Goal : Data fitting using the start point found by global detection
%
%-------------------------------------------------------------------------%
%
% inputs: 
% hol           = hologram
% found         = coordinates of found particles roughly found by global detection , 
%                 [x(in pixels),y(in pixels),z(in meters),r(in meters)]
% lambda        = wavelength in meters
% pixel         = pixel size in meters
% w1            = weighting matrix or Mask
% FOV_size1 
% & FOV_size2   = indexes of pixels in the hologram which contain the original hologram's data
% I0            = mean of hologram if it is removed in advance
% K             = fill-factor of camera
% factors       = [dx dz dr corf itter] options of optimization, 
%               dx=minimum change on parameter x in meters , dz=minimum change on parameter z in meters,
%               dr=minimum change on parameter r in meters,coef is multiplied to CRLB to stop optimization,
%               itter=maximum iteration number
% ranges        = [Zmin Zmax;Rmin Rmax]
% SecondT       = 1 if second term of EQ. should be counted in
% MEXmodelFAST  = 1 if model with multi-threading is required
%-------------------------------------------------------------------------%
% outputs:
% precise       = x(in pixel),y(in pixel),z(in meter),r(in meter) coordinates of particles after local optimization
%
%-------------------------------------------------------------------------%
%
% To test:
% n=5; lambda=0.532e-6; pixel=7e-6; holsize=[1024 1024]; I0= 5;
% SNR=100; K=sqrt(0.7);SIGMA=I0/SNR;
% ranges=[0.2 0.25;50e-6 80e-6]; coordinates=0;
% meanh=0; mrate=[1 1];
% [hol,particle,Mmean,noise]=simulate_holo_mex(n,lambda,pixel,holsize(1),holsize(2),I0,SNR,ranges(1,:),ranges(2,:),1,coordinates,meanh,K);
% found(1,1:2)=(particle(1,1:2)+1)*pixel; found(1,3)=particle(1,3)+0.005;
% if (particle(1,4)+1e-6<ranges(2,2))
% found(1,4)=particle(1,4)+1e-6; else
% found(1,4)=particle(1,4)-1e-6;end
% w1=ones(holsize);FOV_size1=1:holsize(1);FOV_size2=1:holsize(2);
% [dx,dy,dz,dr]=CRLB_point(found(1,1)*pixel,found(1,2)*pixel,found(1,3),found(1,4),pixel,lambda,holsize,mrate,SIGMA,K,0); 
% factors=[dx dz dr 100 15];SecondT=0;
% precise=local_optim(hol,found,lambda,pixel,w1,FOV_size1,FOV_size2,I0,K,factors,ranges,SecondT,1);
% precise(:,1:2)=precise(:,1:2)/pixel;
%-------------------------------------------------------------------------%
%!-------------------------------------------------------------------------%
%! processing time : ~21.325 s on Cetautomatix .  
%! profiler : local_optim=0.009s, model_calc_faster=2.017s (80 calls)
%! local_optim_difference=6.905s (80 calls)
%-------------------------------------------------------------------------%
% functions used :  model_calc_faster.cpp
%-------------------------------------------------------------------------%
% [1] F. Soulez, L. Denis, E. Thiebaut, C. Fournier, et C. Goepfert, "Inverse problem approach in particle digital holography:
%  out-of-field particle detection made possible", Journal of the Optical Society of America A, vol. 24, no. 12, p. 3708-3716, 2007
%-------------------------------------------------------------------------%

%% checking the inputs
if nargin~=13
    errordlg('Error ! Check the number of inputs. It should be 12','Error');

    'Error ! Check the number of inputs. It should be 12'

elseif (size(hol,1)<2 || size(hol,2)<2)
    errordlg('Error ! 1rst input: Width and length of the hologram should be bigger than 1!','Error');
    'Error ! 1rst input: Width and length of the hologram should be bigger than 1!'
    
elseif (size(found,1)<2 && size(found,2)<2)
    errordlg('Error ! 2nd input: No initial points has been provided to the optimization process!','Error');
    'Error ! 2nd input: No initial points has been provided to the optimization process!'
elseif lambda > 1e-3
    errordlg('Error ! 3rd input: the wavelenght of laser should be smaller than 0.001 meter!','Error');
    'Error ! 3rd input: the wavelenght of laser should be smaller than 0.001 meter!'
 elseif lambda < 0
    errordlg('Error ! 3rd input: the wavelenght of laser should be positive!','Error');
    'Error ! 3rd input: the wavelenght of laser should be positive!'
    
elseif pixel>1e-2
    errordlg('Error ! 4th input: The camera s pixel size is irrationaly big!','Error');
    'Error ! 4th input: The camera s pixel size is irrationaly big!'  
 
elseif (size(w1,1)~=size(hol,1) || size(w1,2)~=size(hol,2) )
     errordlg('Error ! 5th input: The provided weighting matrix should be the same size as the first input!','Error');
    'Error ! 5th input: The provided weighting matrix should be the same size as the first input!'

elseif (size(FOV_size1,2)<2 || size(FOV_size2,2)<2 )
     errordlg('Error ! 6th or 7th input: The field of view should be larger than 1 pixel in width or height! ','Error');
    'Error ! 6th or 7th input: The field of view should be larger than 1 pixel in width or height!'

 elseif (numel(I0)~=1)
     errordlg('Error ! 8th input: Provide the I0 as a single value representing the magnitude of source laser','Error');
    'Error ! 8th input: Provide the I0 as a single value representing the magnitude of source laser'
    
elseif (K<0)
     errordlg('Error ! 9th input: The fill-factor or the rate of down-sampling should be nonnegative! ','Error');
    'Error ! 9th input: The fill-factor or the rate of down-sampling should be nonnegative!'

elseif (size(factors,2)~=5 )
     errordlg('Error ! 10th input: This input should contain [dx dz dr coef #iter]! ','Error');
    'Error ! 10th input: This input should contain [dx dz dr coef #iter]'
    
elseif (size(ranges,1)~=2 || size(ranges,2)~=2 )
     errordlg('Error ! 11th input: This input should contain [Zmin Zmax;Rmin Rmax]! ','Error');
    'Error ! 11th input: This input should contain [Zmin Zmax;Rmin Rmax]!'
    
elseif (numel(SecondT)~=1 )
     errordlg('Error ! 12th input: This input should contain a single! ','Error');
    'Error ! 12th input: The field of view should be larger than 1 pixel in width or height!'
    

%% begin the calculations    
else


map_size=size(hol);

if (factors(1)~=0)
    factorR=factors(3)/factors(1);
    omega=factors(2)/factors(1);
else
     factorR=0.5660;
end


[n l]=size(found);        % found comes from global detection. It is n*4 in the case of n particles
                          % - [x y z r]

 ws=sum(sum(w1(FOV_size1,FOV_size2)));

for i=1:n % n is the number of particles, Here it is equal to 1
    if (factors(1)==0) 
        omega=pixel*map_size(1)/2/found(i,3)/1.5;
    end
     found_c=found(i,1:4);
     found_c(3)=found_c(3)/omega;
     found_c(4)=found_c(4)/factorR;
    % using jacobian, putting restriction on input values, output of
    % finction, maximum number of itterations, maximum number of function
    % evaluations
    
     options = optimset('Jacobian','off','TolX',factors(4)*factors(1),'TolFun',1e-40,'MaxIter',factors(5),'Display','iter','MaxFunEvals',10^50);%optimset('OutputFcn', @outfun_nested,'Jacobian','off','TolX',2*dx,'TolFun',1e-40,'MaxIter',50,'Display','iter','MaxFunEvals',10^50);
     
     %local optimization
     precise(i,1:4)= lsqnonlin(@difference,found_c,[-map_size(1)/2*pixel -map_size(2)/2*pixel ranges(1,1)/omega ranges(2,1)/factorR],[(map_size(1)/2-1)*pixel (map_size(2)/2-1)*pixel ranges(1,2)/omega ranges(2,2)/factorR],options);   
 
     precise(i,3)=precise(i,3)*omega;
     precise(i,4)=precise(i,4)*factorR;

 

end
 end
%% 
    function [R]=difference(found_k) % input is the found particle. Outputs are the cost function and the jacobian
    %
    % inputs:
    % found_k       = coordinates of particle during the current step of optimization
    %
    % outputs:
    % R             = vector form of each residu before being squared and summed up to form the cost function
    % JJ            = vector form of the jacobian
   mm=sum(sum(hol(FOV_size1,FOV_size2).*w1(FOV_size1,FOV_size2)))/ws;    
   hol(FOV_size1,FOV_size2) = hol(FOV_size1,FOV_size2)-mm;

    
    % produce model - second output is the values for J function

if (MEXmodelFAST==1)
    g_current=model_calc_faster(found_k(1,1),found_k(1,2),found_k(1,3)*omega,found_k(1,4)*factorR,pixel,lambda,map_size,0,K);
else
        [g_current dummy jini]=model_mex(found_k(1,1),found_k(1,2),found_k(1,3)*omega,found_k(1,4)*factorR,pixel,lambda,map_size,0,K);  
%     clear dummy
end
    wg=sum(sum(w1.*g_current)); % wg
    wrg=sum(sum(w1.*hol.*g_current)); %wrg
    wg2=sum(sum(w1.*(g_current.^2)));%wg2
    wr=sum(sum(w1.*hol));
    alphall=ws.*(wg.*wr-wrg)/(ws.*wg2 - wg.^2);
    I_p=(wg.^2.*wr-wg.*wrg)./(ws.*wg2 - wg.^2);

    
    % creating the cost function
    pp=zeros(map_size(1),map_size(2));
    if (SecondT==1 && MEXmodelFAST~=1)
        I00=I0+I_p+mm;
        pp(FOV_size1,FOV_size2) = hol(FOV_size1,FOV_size2) - ( I_p - alphall.* g_current(FOV_size1,FOV_size2)+I00*(alphall/2/I00).^2.*( (found_k(1,4)*factorR)^2 *pi/lambda/ (found_k(1,3)*omega) )^2*(jini(FOV_size1,FOV_size2).^2)) ;
    else
        pp(FOV_size1,FOV_size2) = hol(FOV_size1,FOV_size2) - ( I_p - alphall.* g_current(FOV_size1,FOV_size2));
    end
    R=reshape(pp,[map_size(1)*map_size(2) 1]);
    

    end
end
